@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,8 +23,9 @@
 #include <config.h>
 #endif
 
-#include <string.h>
+#include <stdio.h>
 #include <ctype.h>
+#include <string.h>
 
 #include <glib.h>
 
@@ -48,7 +49,7 @@ void g_at_util_debug_chat(gboolean in, const char *str, gsize len,
 	for (i = 0; i < len; i++) {
 		char c = str[i];
 
-		if (isprint(c))
+		if (g_ascii_isprint(c))
 			escaped += 1;
 		else if (c == '\r' || c == '\t' || c == '\n')
 			escaped += 2;
@@ -94,7 +95,7 @@ void g_at_util_debug_chat(gboolean in, const char *str, gsize len,
 			escaped += esc_size;
 			break;
 		default:
-			if (isprint(c))
+			if (g_ascii_isprint(c))
 				escaped_str[escaped++] = c;
 			else {
 				escaped_str[escaped++] = '\\';
@@ -132,6 +133,48 @@ void g_at_util_debug_dump(gboolean in, const unsigned char *buf, gsize len,
 	g_string_free(str, TRUE);
 }
 
+void g_at_util_debug_hexdump(gboolean in, const unsigned char *buf, gsize len,
+				GAtDebugFunc debugf, gpointer user_data)
+{
+	static const char hexdigits[] = "0123456789abcdef";
+	char str[68];
+	gsize i;
+
+	if (debugf == NULL || !len)
+		return;
+
+	str[0] = in ? '<' : '>';
+
+	for (i = 0; i < len; i++) {
+		str[((i % 16) * 3) + 1] = ' ';
+		str[((i % 16) * 3) + 2] = hexdigits[buf[i] >> 4];
+		str[((i % 16) * 3) + 3] = hexdigits[buf[i] & 0xf];
+		str[(i % 16) + 51] = g_ascii_isprint(buf[i]) ? buf[i] : '.';
+
+		if ((i + 1) % 16 == 0) {
+			str[49] = ' ';
+			str[50] = ' ';
+			str[67] = '\0';
+			debugf(str, user_data);
+			str[0] = ' ';
+		}
+	}
+
+	if (i % 16 > 0) {
+		gsize j;
+		for (j = (i % 16); j < 16; j++) {
+			str[(j * 3) + 1] = ' ';
+			str[(j * 3) + 2] = ' ';
+			str[(j * 3) + 3] = ' ';
+			str[j + 51] = ' ';
+		}
+		str[49] = ' ';
+		str[50] = ' ';
+		str[67] = '\0';
+		debugf(str, user_data);
+	}
+}
+
 gboolean g_at_util_setup_io(GIOChannel *io, GIOFlags flags)
 {
 	GIOFlags io_flags;
