@@ -305,18 +305,25 @@ static gboolean setup_huawei(struct modem_info *modem)
 
 		if (g_strcmp0(info->label, "modem") == 0 ||
 				g_strcmp0(info->interface, "255/1/1") == 0 ||
-				g_strcmp0(info->interface, "255/2/1") == 0) {
+				g_strcmp0(info->interface, "255/2/1") == 0 ||
+				g_strcmp0(info->interface, "255/1/49") == 0) {
 			mdm = info->devnode;
 		} else if (g_strcmp0(info->label, "pcui") == 0 ||
 				g_strcmp0(info->interface, "255/1/2") == 0 ||
-				g_strcmp0(info->interface, "255/2/2") == 0) {
+				g_strcmp0(info->interface, "255/2/2") == 0 ||
+				g_strcmp0(info->interface, "255/1/50") == 0) {
 			pcui = info->devnode;
 		} else if (g_strcmp0(info->label, "diag") == 0 ||
 				g_strcmp0(info->interface, "255/1/3") == 0 ||
-				g_strcmp0(info->interface, "255/2/3") == 0) {
+				g_strcmp0(info->interface, "255/2/3") == 0 ||
+				g_strcmp0(info->interface, "255/1/51") == 0) {
 			diag = info->devnode;
-		} else if (g_strcmp0(info->interface, "255/1/8") == 0) {
+		} else if (g_strcmp0(info->interface, "255/1/8") == 0 ||
+				g_strcmp0(info->interface, "255/1/56") == 0) {
 			net = info->devnode;
+		} else if (g_strcmp0(info->interface, "255/1/9") == 0 ||
+				g_strcmp0(info->interface, "255/1/57") == 0) {
+			qmi = info->devnode;
 		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
 			if (g_strcmp0(info->number, "00") == 0)
 				mdm = info->devnode;
@@ -331,9 +338,15 @@ static gboolean setup_huawei(struct modem_info *modem)
 		}
 	}
 
+	if (qmi != NULL && net != NULL) {
+		ofono_modem_set_driver(modem->modem, "gobi");
+		goto done;
+	}
+
 	if (mdm == NULL || pcui == NULL)
 		return FALSE;
 
+done:
 	DBG("mdm=%s pcui=%s diag=%s qmi=%s net=%s", mdm, pcui, diag, qmi, net);
 
 	ofono_modem_set_string(modem->modem, "Device", qmi);
@@ -621,6 +634,38 @@ static gboolean setup_telit(struct modem_info *modem)
 	return TRUE;
 }
 
+static gboolean setup_he910(struct modem_info *modem)
+{
+	const char *mdm = NULL, *aux = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+
+		DBG("%s %s %s %s", info->devnode, info->interface,
+						info->number, info->label);
+
+		if (g_strcmp0(info->interface, "2/2/1") == 0) {
+			if (g_strcmp0(info->number, "00") == 0)
+				mdm = info->devnode;
+			else if (g_strcmp0(info->number, "06") == 0)
+				aux = info->devnode;
+		}
+	}
+
+	if (aux == NULL || mdm == NULL)
+		return FALSE;
+
+	DBG("modem=%s aux=%s", mdm, aux);
+
+	ofono_modem_set_string(modem->modem, "Modem", mdm);
+	ofono_modem_set_string(modem->modem, "Aux", aux);
+
+	return TRUE;
+}
+
 static gboolean setup_simcom(struct modem_info *modem)
 {
 	const char *mdm = NULL, *aux = NULL, *gps = NULL, *diag = NULL;
@@ -765,6 +810,7 @@ static struct {
 	{ "novatel",	setup_novatel	},
 	{ "nokia",	setup_nokia	},
 	{ "telit",	setup_telit	},
+	{ "he910",	setup_he910	},
 	{ "simcom",	setup_simcom	},
 	{ "zte",	setup_zte	},
 	{ "icera",	setup_icera	},
@@ -946,7 +992,7 @@ static struct {
 	{ "icera",	"cdc_acm",	"0421", "0633"	},
 	{ "icera",	"cdc_ether",	"0421", "0633"	},
 	{ "mbm",	"cdc_acm",	"0bdb"		},
-	{ "mbm"		"cdc_ether",	"0bdb"		},
+	{ "mbm",	"cdc_ether",	"0bdb"		},
 	{ "mbm",	"cdc_acm",	"0fce"		},
 	{ "mbm",	"cdc_ether",	"0fce"		},
 	{ "mbm",	"cdc_acm",	"413c"		},
@@ -962,6 +1008,7 @@ static struct {
 	{ "sierra",	"sierra_net"			},
 	{ "option",	"option",	"0af0"		},
 	{ "huawei",	"option",	"201e"		},
+	{ "huawei",	"cdc_wdm",	"12d1"		},
 	{ "huawei",	"cdc_ether",	"12d1"		},
 	{ "huawei",	"qmi_wwan",	"12d1"		},
 	{ "huawei",	"option",	"12d1"		},
@@ -974,6 +1021,7 @@ static struct {
 	{ "simcom",	"option",	"05c6", "9000"	},
 	{ "telit",	"usbserial",	"1bc7"		},
 	{ "telit",	"option",	"1bc7"		},
+	{ "he910",	"cdc_acm",	"1bc7", "0021"	},
 	{ "nokia",	"option",	"0421", "060e"	},
 	{ "nokia",	"option",	"0421", "0623"	},
 	{ "samsung",	"option",	"04e8", "6889"	},
